@@ -5,7 +5,7 @@
 #
 #    Copyright (C) 2004  Richard Jones  <richard followed by funny at sign then jones then a dot then name>
 #
-#    GmailFS - Gmail Filesystem Version 0.7.2
+#    GmailFS - Gmail Filesystem Version 0.8.0
 #    This program can be distributed under the terms of the GNU GPL.
 #    See the file COPYING.
 #
@@ -17,12 +17,16 @@ GmailFS provides a filesystem using a Google Gmail account as its storage medium
 #@+others
 #@+node:imports
 
+import fuse
 from fuse import Fuse
 import os
 from errno import *
 from stat import *
 from os.path import abspath, expanduser
 
+fuse.fuse_python_api = (0, 2)
+
+
 import thread
 import quopri
 import libgmail
@@ -89,10 +93,8 @@ def GmailConfig(fname):
     global References
     global DefaultUsername, DefaultPassword, DefaultFsname
     global NumberQueryRetries
-    try:
-      cp.read(fname)
-    except:
-      log.warning("Unable to read configuration file: " + fname)
+    if cp.read(fname) == []:
+      log.warning("Unable to read configuration file: " + str(fname))
       return
 
     sections = cp.sections()
@@ -162,6 +164,10 @@ class reference_class:
 # old filesystems were stored with 7bit encodings
 # but new ones are all quoted printable
 def fixQuotedPrintable(body):
+    # first remove headers
+    newline = body.find("\r\n\r\n")
+    if newline >= 0:
+	body = body[newline:]
     fixed = body
     if re.search("Content-Transfer-Encoding: quoted",body):
         fixed = quopri.decodestring(body)
@@ -502,7 +508,9 @@ class OpenFile:
         
         log.debug("about to try and find inode:"+str(self.inode.ino)+" blocknumber:"+str(readblock))
         if self.lastBlockRead == readblock:
-            return self.lastBlockBuffer
+            contentList = list(" "*self.blocksize)
+	    contentList[0:] = self.lastBlockBuffer
+            return contentList
         ga = self.inode.ga
         subject = 'b='+str(self.inode.ino)+' x='+str(readblock)+''
         folder = _getMessagesByQuery(ga,'subject:'+subject)
@@ -525,7 +533,6 @@ class OpenFile:
         self.lastBlockBuffer = a.content
         contentList = list(" "*self.blocksize)
         contentList[0:] = a.content
-
         return contentList
 
 #@-node:class OpenFile
@@ -537,12 +544,11 @@ class Gmailfs(Fuse):
 
     
     #@+node:__init__
-    def __init__(self, mountpoint, **kw):
-
-        Fuse.__init__(self, mountpoint, **kw)
-    
-        log.info("Mountpoint: %s" % self.mountpoint)
-        log.info("Unnamed mount options: %s" % (self.optlist,))
+    def __init__(self, extraOpts, mountpoint, *args, **kw):
+        Fuse.__init__(self, *args, **kw)
+        self.fuse_args.mountpoint = mountpoint
+        self.optdict = extraOpts
+        log.info("Mountpoint: %s" % mountpoint)
 	# obfuscate sensitive fields before logging
 	loggableOptdict = self.optdict.copy()
 	loggableOptdict['password'] = '*' * 8
@@ -626,8 +632,22 @@ class Gmailfs(Fuse):
     
     #@-node:attribs
     
+    class GmailStat(fuse.Stat):
+        def __init__(self):
+            self.st_mode = 0
+            self.st_ino = 0
+            self.st_dev = 0
+            self.st_nlink = 0
+            self.st_uid = 0
+            self.st_gid = 0
+            self.st_size = 0
+            self.st_atime = 0
+            self.st_mtime = 0
+            self.st_ctime = 0
+
     #@+node:getattr
     def getattr(self, path):
+        st = Gmailfs.GmailStat();
         log.debug("get stat:"+path)
         #st_mode (protection bits)
         #st_ino (inode number)
@@ -652,12 +672,22 @@ class Gmailfs(Fuse):
         if inode:
             if log.isEnabledFor(logging.DEBUG):
                 log.debug("inode "+str(inode))
-            statTuple = (inode.mode,inode.ino,inode.dev,inode.nlink,inode.uid,
-                         inode.gid,inode.size,inode.atime,inode.mtime,
-                         inode.ctime)
+            st.st_mode = inode.mode
+            st.st_ino = inode.ino
+            st.st_dev = inode.dev
+            st.st_nlink = inode.nlink
+            st.st_uid = inode.uid
+            st.st_gid = inode.gid
+            st.st_size = inode.size
+            st.st_atime = inode.atime
+            st.st_mtime = inode.mtime
+            st.st_ctme = inode.ctime
+#            statTuple = (inode.mode,inode.ino,inode.dev,inode.nlink,inode.uid,
+#                         inode.gid,inode.size,inode.atime,inode.mtime,
+#                         inode.ctime)
             if log.isEnabledFor(logging.DEBUG):
-                log.debug("statsTuple "+str(statTuple))
-            return statTuple
+                log.debug("statsTuple "+str(st))
+            return st
         else:
             e = OSError("No such file"+path)
             e.errno = ENOENT
@@ -679,9 +709,11 @@ class Gmailfs(Fuse):
         return m.group(1)
     #@-node:readlink
     
-    #@+node:getdir
-    def getdir(self, path):
+
+   #@+node:readdir
+    def readdir(self, path, offset):
         try:
+            log.debug("at top of readdir");
             log.debug("getting dir "+path)
             fspath = _pathSeparatorEncode(path)
             log.debug("querying for:"+''+PathNameTag+'='+PathStartDelim+
@@ -722,7 +754,9 @@ class Gmailfs(Fuse):
         except:
             _logException("got exception when getmessages")
             lst = None
-        return map(lambda x: (x,0), lst)
+        #return map(lambda x: (x,0), lst)
+        for r in lst:
+	    yield fuse.Direntry(r)
     #@-node:getdir
     
     #@+node:unlink
@@ -1033,6 +1067,7 @@ class Gmailfs(Fuse):
         Feel free to set any of the above values to 0, which tells
         the kernel that the info is not available.
         """
+        st = fuse.StatVfs()
         block_size = 1024
         quotaInfo = self.ga.getQuotaInfo()
         quotaMbUsed = quotaInfo[QU_SPACEUSED]
@@ -1046,7 +1081,14 @@ class Gmailfs(Fuse):
         files = 0
         files_free = 0
         namelen = 80
-        return (block_size, blocks, blocks_free, blocks_avail, files, files_free, namelen)
+        st.f_bsize = block_size
+	st.f_frsize = block_size
+	st.f_blocks = blocks
+	st.f_bfree = blocks_free
+	st.f_bavail = blocks_avail
+	st.f_files = files
+	st.f_ffree = files_free
+	return st
     #@-node:statfs
 
     #@+node:fsync
@@ -1120,16 +1162,18 @@ defaultLogFormatter = logging.Formatter("%(asctime)s %(levelname)-10s %(message)
 defaultLoggingHandler = logging.StreamHandler(sys.stdout)
 _addLoggingHandlerHelper(defaultLoggingHandler)
 
-GmailConfig("/etc/gmailfs.conf")
+GmailConfig(['/etc/gmailfs.conf',os.path.expanduser('~/.gmailfs')])
 try:
     libgmail.ConfigLogs(log)
 except:
     pass
 
 def main(mountpoint, namedOptions):
-    server = Gmailfs(mountpoint, **namedOptions)
+    server = Gmailfs(namedOptions,mountpoint,version="gmailfs 0.8.0",usage='',dash_s_do='setsingle')
+#    server.parser.mountpoint = mountpoint
+    server.parse(errex=1)
     server.flags = 0
-    server.multithreaded = 0;
+    server.multithreaded = False;
     server.main()
 
 if __name__ == '__main__':
